use ring::signature::Ed25519KeyPair;
use sha2::Digest;
use crate::schema;
use microrm::prelude::*;

pub struct CertStore { }

impl CertStore {
    pub fn new() -> Self {
        Self { }
    }

    fn realm_id(&self, qi: &microrm::QueryInterface, realm_name: &str) -> Option<schema::RealmID> {
        qi.get().by(schema::Realm::Shortname, realm_name).one().ok()?.map(|v| v.id())
    }

    pub fn generate_in(&self, qi: &microrm::QueryInterface, realm_name: &str) -> Result<String, &'static str> {
        let mut rng = ring::rand::SystemRandom::new();
        let sign_generated = Ed25519KeyPair::generate_pkcs8(&mut rng);

        let realm_id = self.realm_id(qi, realm_name).ok_or("Failed to find realm")?;

        if let Err(_) = sign_generated {
            return Err("Failed to generate key")
        }
        let sign_generated = sign_generated.unwrap();

        let keydata = sign_generated.as_ref().to_owned();

        let mut key_hasher = sha2::Sha256::new();
        key_hasher.update(&keydata);
        let mut key_id = base64::encode(key_hasher.finalize());
        key_id.truncate(16);

        qi.add(&schema::Key { realm: realm_id, key_id: key_id.clone(), keydata }).expect("Couldn't add key");

        Ok(key_id)
    }
}

pub fn inspect(db: &microrm::DB, realm_name: &str) {
    let qi = db.query_interface();

    let realm = qi.get().by(schema::Realm::Shortname, realm_name).one().expect("couldn't load realm");
    if realm.is_none() {
        println!("No such realm {}", realm_name);
        return
    }
    let realm = realm.unwrap();

    println!("Retrieving keys for {} realm...", realm_name);
    let keys = qi.get().by(schema::Key::Realm, &realm.id()).all().expect("couldn't load keys");
    for key in keys {
        println!("- [{:20}]", key.key_id);
    }
}

pub fn generate(db: &microrm::DB, realm_name: &str) {
    let cs = CertStore::new();
    let qi = db.query_interface();
    if let Err(e) = cs.generate_in(&qi, realm_name) {
        println!("Failed to generate key: {}", e);
    }
}
