use serde::{Serialize,Deserialize};
pub use microrm::{Entity, Schema};

/// Top-level partitioning object
#[derive(Entity,Serialize,Deserialize)]
pub struct Realm {
    pub shortname: String
}

/// Cryptographic signing key
#[derive(Entity,Serialize,Deserialize)]
pub struct Key {
    pub realm: RealmID,
    pub key_id: String,
    #[serde(with = "serde_bytes")]
    pub keydata: Vec<u8>
}

/// End-user representation object
#[derive(Entity,Serialize,Deserialize)]
pub struct User {
    pub realm: RealmID,
    pub username: String,
}

/// End-user authentication
#[derive(Entity,Serialize,Deserialize)]
pub struct Password {
    pub user: UserID,
    pub salt: String,
    pub hash: String
}

/// User semantic grouping
#[derive(Entity,Serialize,Deserialize)]
pub struct Group {
    pub realm: RealmID,
    pub shortname: String,
}

/// User membership in group
#[derive(Entity,Serialize,Deserialize)]
pub struct GroupMembership {
    pub group: GroupID,
    pub user: UserID,
}

/// OAuth2 client representation
#[derive(Entity,Serialize,Deserialize)]
pub struct Client {
    pub realm: RealmID,
    pub shortname: String,
    pub secret: String,
}

/// Requested group of permissions
#[derive(Entity,Serialize,Deserialize)]
pub struct Scope {
    pub realm: RealmID,
    pub shortname: String,
}

/// Specific atomic permission
#[derive(Entity,Serialize,Deserialize)]
pub struct Role {
    pub realm: RealmID,
    pub shortname: String,
}

/// Role membership in scope
#[derive(Entity,Serialize,Deserialize)]
pub struct ScopeRoles {
    pub scope: ScopeID,
    pub role: RoleID,
}

/// Assigned permissions in group
#[derive(Entity,Serialize,Deserialize)]
pub struct GroupRoles {
    pub scope: ScopeID,
    pub role: RoleID,
}


#[derive(Entity,Serialize,Deserialize)]
pub struct RevokedTokens {
    pub user: UserID,
    pub nonce: String,
}

pub fn schema() -> Schema {
    Schema::new()
        .add::<Realm>()
        .add::<Key>()
        .add::<User>()
        .add::<Password>()
        .add::<Group>()
        .add::<GroupMembership>()
        .add::<Client>()
        .add::<Scope>()
        .add::<Role>()
        .add::<ScopeRoles>()
        .add::<GroupRoles>()
}
